#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

##############################################################################################
# Copyright (c) by George Ruinelli 
# License: 	GPL3
# Description:	Basic bible viewer
##############################################################################################


import sys
from PySide.QtCore import *
from PySide.QtGui import *
from PySide.QtDeclarative import *
from PySide.QtGui import QDesktopServices as QDS

import random
import math
import string
import platform
import os
import ConfigParser

import parser
import modulemanager
import data

   
    
####################################################################################################
class Configuration():
    def __init__(self):
	data.configpath = os.path.join(QDS.storageLocation(QDS.DataLocation), "MiniBible")
	data.configfile = data.configpath + "/" + data.configfile
  
	  
    def Load(self):
	print "Loading configuration from:", data.configfile
	self.ConfigParser = ConfigParser.SafeConfigParser()
	try:
	    self.ConfigParser.read(data.configfile)
	except: #use default config
	    print "Configuration file "+ data.configfile + " not existing or not compatible"

	try:
	    self.ConfigParser.add_section('main')
	except:
	    pass

	try:
	    data.translation = self.ConfigParser.get("main", "translation")  
	    data.second_translation = self.ConfigParser.get("main", "second_translation")  
	    data.book = self.ConfigParser.get("main", "book")   
	    data.chapter = self.ConfigParser.get("main", "chapter")   
	    data.font_sizes_index = int(self.ConfigParser.get("main", "font_sizes_index"))
	    data.book_languages_index = int(self.ConfigParser.get("main", "book_languages_index"))  
	    data.use_second_translation = self.ConfigParser.getboolean("main", "use_second_translation") 
	    data.theme_inverted = self.ConfigParser.getboolean("main", "theme_inverted")   
	    print "Configuration loaded"  
	except:
	    print "Error loading configuration, using default value"
  

	
    
    
    
    
    
    def Write(self):
	print "Write configuration to:", data.configfile  
	
	#self.ConfigParser.add_section('main')
		  
	self.ConfigParser.set('main', 'translation', str(data.translation))
	self.ConfigParser.set('main', 'second_translation', str(data.second_translation))
	self.ConfigParser.set('main', 'book', str(data.book))
	self.ConfigParser.set('main', 'chapter', str(data.chapter))
	self.ConfigParser.set('main', 'font_sizes_index', str(data.font_sizes_index))
	self.ConfigParser.set('main', 'book_languages_index', str(data.book_languages_index))
	self.ConfigParser.set('main', 'use_second_translation', str(data.use_second_translation))
	self.ConfigParser.set('main', 'theme_inverted', str(data.theme_inverted))

	try:
	    os.makedirs(data.configpath)
	except:
	  pass

	try:
	    handle = open(data.configfile, 'w')
	    self.ConfigParser.write(handle)
	    handle.close()
	    print "Configuration saved"
	except:
	    print "Failed to write configuration file!"

  
  
  
  
  
  
####################################################################################################
class ModuleConfiguration():
    def __init__(self, translation):   
	self.moduleconfigfile = data.modules_config_path + "/" + translation + ".conf"  
  
	print "Loading module configuration from:", self.moduleconfigfile
	self.ConfigParser = ConfigParser.SafeConfigParser()
	try:
	    self.ConfigParser.read(self.moduleconfigfile)
	except: #use default config
	    print "Module configuration file "+ self.moduleconfigfile + " not existing or not compatible. Will not be able to show any module information"
	    return
  
	sections = self.ConfigParser.sections()
	
	self.Lang = ""
	self.Version = ""
	self.Description = ""
	self.About = ""
	self.DistributionLicense = ""
	self.ShortCopyright = ""
	self.Encoding = ""
	self.ShortPromo = ""
	self.CipherKey = ""
	self.Encoding = "Latin-1"
	
	
	
	try:
	    self.Lang = self.ConfigParser.get(sections[0], "Lang")  
	except:
	    pass
	try:
	    self.Version = self.ConfigParser.get(sections[0], "Version")   
	except:
	    pass
	try:
	    self.Description = self.ConfigParser.get(sections[0], "Description")   
	except:
	    pass
	try:
	    self.About = self.ConfigParser.get(sections[0], "About")   
	except:
	    pass
	try:
	    self.DistributionLicense = self.ConfigParser.get(sections[0], "DistributionLicense")   
	except:
	    pass
	try:
	    self.ShortCopyright = self.ConfigParser.get(sections[0], "ShortCopyright") 
	except:
	    pass
	try:   
	    self.Encoding = self.ConfigParser.get(sections[0], "Encoding")    
	except:
	    pass
	try:
	    self.ShortPromo = self.ConfigParser.get(sections[0], "ShortPromo")  
	except:
	    pass
	try:  
	    self.CipherKey = self.ConfigParser.get(sections[0], "CipherKey")  
	except:
	    pass
	try:  
	    self.Encoding = self.ConfigParser.get(sections[0], "Encoding")  
	except:
	    pass
  
  
	#print "=============================\nModule information:"
	#print "Module:", translation 
	#print "Language:", self.Lang
	#print "Description:", self.Description
	#print "About:", self.About
	#print "DistributionLicense:", self.DistributionLicense
	#print "ShortCopyright:", self.ShortCopyright
	#print "Encoding:", self.Encoding
	#print "ShortPromo:", self.ShortPromo
	#print "CipherKey:", self.CipherKey
	#print "Encoding:", self.Encoding   
	#print "============================="
  
  
  
  
    
####################################################################################################

class QML_to_Python_Interface(QObject):
    @Slot(str)
    def setTranslation(self, translation):
	data.translation = translation
	loadTranslation()
	chapters = module.number_of_chapters_in_book(data.book) 
	#fillListOfBooks()
	root.loadParametersfromPython(data.translation, data.second_translation, data.book, chapters, data.chapter, data.book_languages_index, data.font_sizes_index)
	fillBibleText()
	
	
    @Slot(str)
    def setTranslation2(self, translation):
	data.second_translation = translation
	loadTranslation()
	chapters = module.number_of_chapters_in_book(data.book) 
	#fillListOfBooks()
	root.loadParametersfromPython(data.translation, data.second_translation, data.book, chapters, data.chapter, data.book_languages_index, data.font_sizes_index)
	data.use_second_translation = True
	fillBibleText()
      
      		
    @Slot()
    def toggleSplitScreen(self):
	if(data.use_second_translation == True): data.use_second_translation = False
	else: data.use_second_translation = True
	fillBibleText()
      
  
    @Slot(str)
    def setBook(self, book):
	data.book = book
	data.chapter = 1	
	chapters = module.number_of_chapters_in_book(data.book)    
	root.loadParametersfromPython(data.translation, data.second_translation, data.book, chapters, data.chapter, data.book_languages_index, data.font_sizes_index)
	fillBibleText()


    @Slot(str)
    def setChapter(self, chapter):
	data.chapter = chapter	
	fillBibleText()


    @Slot(int)
    def setFontSize(self, fontSize):
	data.font_sizes_index = int(fontSize)
	fillBibleText()

    #@Slot(float)
    #def setFontSizeScale(self, fontSizeScale):
	##data.font_sizes_index = int(fontSize)
	#a =data.text_font_size
	#delta = fontSizeScale - 1
	#delta = delta * 0.5
	#data.text_font_size = int(data.text_font_size * (1+delta))
	#print "Old:", a, "New:", data.text_font_size, delta
	#if(data.text_font_size < 10): data.text_font_size=16
	#if(data.text_font_size > 50): data.text_font_size=50
	#fillBibleText()
	
	
    @Slot(result=str)
    def getVersion(self):
	txt = str(data.version) + "." + str(data.build)
	return txt
	

    @Slot(str)
    def setBookLanguage(self, book_languages_index):
	data.book_languages_index = int(book_languages_index)
	print "Switch book language to:", data.book_languages[data.book_languages_index]
	#fillListOfBooks()
	fillBibleText()
	
    @Slot()
    def themeInverted(self):
	if(data.theme_inverted == False): data.theme_inverted = True
	else: data.theme_inverted = False
	print "ThemeInverted:", data.theme_inverted
	root.themeInverted(data.theme_inverted)
	fillBibleText()
	
	
    @Slot()
    def fillListOfInstalledTranslations(self):
	#print "fillListOfInstalledTranslations"
	#installed = modulemanager.get_module_list()	
	translations = get_list_of_translations()	
	print translations
	root.fillListOfInstalledTranslations(translations)
    
    	
    @Slot()
    def fillListOfRemoteTranslations(self):
	#print "fillListOfRemoteTranslations"
	translations = modulemanager.get_module_list()	
	#print translations
	root.fillListOfRemoteTranslations(translations)
    
    
    
    @Slot(str)
    def installTranslation(self, translation):
	#print "installTranslation:", translation
	modulemanager.installTranslation(translation)  
	translations = get_list_of_translations()
	root.loadListOfTranslationsFromPython(translations, data.translation, data.second_translation)    
	#root.fillListOfInstalledTranslations(translations)    


    @Slot(result=int)
    def getInstallTranslationProgress(self):
	return modulemanager.getDownloadProgress()
    
    
    
    @Slot(str)
    def removeTranslation(self, translation):
	#print "removeTranslation:", translation
	modulemanager.removeTranslation(translation) 
	translations = get_list_of_translations()
	root.loadListOfTranslationsFromPython(translations, data.translation, data.second_translation)    
	#root.fillListOfInstalledTranslations(translations)
    
    
    #I couldnt figure out how to send an array to QML, so we send a joined string :(
    @Slot(result=str)
    def getListOfBooks(self):
	#print "getListOfBooks"
	#fillListOfBooks()
	book_names = data.book_names[data.book_languages_index]
  
	s = ""
	for i in range(0, len(book_names)):
	    try:
		book_names[i][0] = book_names[i][0].decode("utf-8")		
	    except:
		pass
	    s = s + book_names[i][0] + "," + book_names[i][1] + ";"
	      
	#print s
	return s

    
    
    
    
    
################################################################################################  
    

def html_header():
    h = data.html_header
    h = string.replace(h, "$TITLE_FONT_SIZE$",str(data.title_font_sizes[data.font_sizes_index]))
    h = string.replace(h, "$TEXT_FONT_SIZE$",str(data.text_font_sizes[data.font_sizes_index]))
    #h = string.replace(h, "$TEXT_FONT_SIZE$", str(data.text_font_size))
    h = string.replace(h, "$VERSE_NUMBER_FONT_SIZE$",str(data.verse_number_font_sizes[data.font_sizes_index]))
    
    if(data.theme_inverted == False):
	h = string.replace(h, "$TITLE_COLOR$",str(data.title_color))
	h = string.replace(h, "$TEXT_COLOR$",str(data.text_color))
	h = string.replace(h, "$VERSE_NUMBER_COLOR$",str(data.verse_number_color))
    else: #theme inversed
	h = string.replace(h, "$TITLE_COLOR$",str(data.title_color_inverted))
	h = string.replace(h, "$TEXT_COLOR$",str(data.text_color_inverted))
	h = string.replace(h, "$VERSE_NUMBER_COLOR$",str(data.verse_number_color_inverted))
    
    #print h
    return h
  
  
  
    
    
def format_bible_text(book, chapter, first_verse, last_verse): #verse numbering starting at 1 (not 0)
    print "Formating text for:", book, chapter, ":", first_verse, "-", last_verse
  
    for i in range(0, len(data.book_names[data.book_languages_index])):
	if(data.book_names[data.book_languages_index][i][1] == book):
	    nicebookname = data.book_names[data.book_languages_index][i][0]
	    break

    try:
	nicebookname = nicebookname.decode("utf-8")
    except:
	pass

    html = html_header()
    
    if(data.use_second_translation == False): #normal, show one translation
	t = str(data.translation)
	html = html + "<p class=title>" + nicebookname + " " + str(chapter) + ":" + str(first_verse) + "-" + str(last_verse) + " (<i>" + t + "</i>)</p><p>"
	for i in range(first_verse, last_verse+1):
	    verse = module.text_for_ref(book, chapter, i)
	    verse = verse.decode(module_config.Encoding)
	    try:
		verse = verse.decode("utf-8")
	    except:
		pass
	    #html = html + "<span class=verse_number><a href="+str(i)+">"+str(i)+"</a></span>" + verse + "\n"
	    html = html + "<span class=verse_number>"+str(i)+"</span>" + verse + "\n"
	
    
    else: #use 2 translations in split screen
	t = str(data.translation)
	t2 = str(data.second_translation)
	if(data.theme_inverted == False): linecolor = data.text_color
	else: linecolor = data.text_color_inverted
	html = html + "<table width=100% cellspacing=0 border=0 cellpadding=0><tr>" + \
		    "<td width=49% align=center>" + "<p class=title><i>" + t + "</i></p></td>" + \
		    "<td width=2%></td>" + \
		    "<td width=49% align=center>" + "<p class=title><i>" + t2 + "</i></p></td>" + \
		"</tr>" + \
		"<tr><td bgcolor=" + linecolor + " height=1><img height=1></td><td><img height=1></td><td bgcolor=" + linecolor + " height=1><img height=1></td></tr>" +\
		"<tr>" + \
		    "<td colspan=3 align=center>" + "<p class=title>" + nicebookname + " " + str(chapter)+ ":" + str(first_verse) + "-" + str(last_verse) + "</p>" + "</td>" + \
		"</tr>"
    
    	for i in range(first_verse, last_verse+1):
	    verse = module.text_for_ref(book, chapter, i)
	    verse = verse.decode(module_config.Encoding)
	    verse = verse.strip()
	    
	    verse2 = second_module.text_for_ref(book, chapter, i)
	    verse2 = verse2.decode(second_module_config.Encoding)

	    html = html + "<tr><td><p><span class=verse_number>"+str(i)+"</span>"
	    html = html + verse
	    html = html + "</p></td></td><td>" + \
			      "<td><p><span class=verse_number>"+str(i)+"</span>"
	    html = html + verse2
	    html = html + "</p></td></tr>"
	html = html + "</table>"
    
    
    html = html + "</p>" + data.html_footer
    
    try:
	html = html.decode("utf-8")    
	#print html
    except:
	print "Could not decode html, use as is!"
    return html
    





def get_list_of_translations():
    if(data.modulePathAvailable == True):
	try:
	    dirs = os.listdir(data.modules_path)
	except:
	    print "Failed to read folder:", data.modules_path
	    dirs = []
	
	if(len(dirs) == 0):
	    print "No translation available"
	    data.translation == ""
	    return []
	#else:
	    #print "Total", len(dirs), "translations available:", dirs
	
	
	index=-1
	index2=-1
	for i in range(0, len(dirs)):
	    if(dirs[i] == data.translation):
		index = i
		#break
	    if(dirs[i] == data.second_translation):
		index2 = i
		#break
	if(index == -1): #selected translation not found in list
	    print "selected translation not found in list:", data.translation
	    print "Select first available one:", dirs[0]
	    data.translation = dirs[0] #select first available one
	if(index2 == -1): #selected translation not found in list
	    data.second_translation = dirs[0] #select first available one
	
	dirs.sort()
	return dirs
    else: return []



def fillBibleText():
    number_of_verses=module.number_of_verses_in_chapter(data.book, data.chapter)    
    text = format_bible_text(data.book, data.chapter, 1, number_of_verses)
    root.loadTextfromPython(text)




def loadTranslation():
  global module, second_module, module_config, second_module_config
  print "Load translation:", data.translation
  #try:
  module = parser.ZModule(data.translation)  
  #except:
      #print "Initialisation of module failed"
      #return
  module_config = ModuleConfiguration(data.translation)
  #if(data.use_second_translation == True):
  print "Load 2nd translation:", data.second_translation
  second_module = parser.ZModule(data.second_translation)
  second_module_config = ModuleConfiguration(data.second_translation)
  
  
  #ToDo: make sure there is no UnicodeDecodeError:
  #text = "<b>Name</b><br>"+module_config.Description + " (<i>" + data.translation + "</i>)<br>"
	 #"<b>Copyright</b><br>"+module_config.ShortCopyright + "<br>"
	 
  #if(data.use_second_translation == True):
      #text = text + "<br><br><b>Name</b><br>"+second_module_config.Description + " (<i>" + data.second_translation + "</i>)<br>"
	 #"<b>Copyright</b><br>"+second_module_config.ShortCopyright 
  text = "Not yet implemented"
  root.fillAboutTranslation(text)
  
  


#def fillListOfBooks():
  #book_names = data.book_names[data.book_languages_index]
  
  
  #for i in range(0, len(book_names)-1):
      #try:
	  #book_names[i][0] = book_names[i][0].decode("utf-8")
      #except:
	  #pass
  
  #root.fillListOfBooks(book_names)





def commandline():
    #global module
    print "Command line parameters:", sys.argv[1:]
    l = len(sys.argv)-1
    if(l == 0):
	#print "No command line parameters"
	pass
    elif(l == 4): #book, chapter, verse (verse list)    
	mod_name, book, chapter, verse = sys.argv[1:]

	found = False
	for i in range(0, len(translations)):
	    if(mod_name == translations[i]):
		#print "found translation"
		found = True
		break
	if(found == False):
	    print "Name of translation not valid!"
	    cli_help()	    	
	data.translation = mod_name	
	
	found = False
	for i in range(0, len(data.book_names[data.book_languages_index])):
	    if(book == data.book_names[data.book_languages_index][i][1]):
		#print "found book"
		found = True
		break
	if(found == False):
	    print "Name of book not valid!"
	    cli_help()	    
	data.book = book
	
	#chapters = module.number_of_chapters_in_book(data.book) 
	#if(chapter <= chapters): data.chapter = chapter
	#else: data.chapter = chapters
	data.chapter = int(chapter)
	print "Translation:", data.translation, ", Book:", data.book, ", chapter:", data.chapter, "(Verse selection is not yet implemented)"

    else:
	cli_help()
	
	
def cli_help():
    print "Possible parameters:"
    print "  minibible <translation> <book> <chapter> <verse>"
    print "  I.e.   esv John 3 16       or      gerneue Gen 20 1-10"
    print "Below is a list of all available translations:"
    for i in range(0, len(translations)):
	print "", translations[i]  
    print "Below is a list of all book names:"
    for i in range(0, len(data.book_names[data.book_languages_index])):
	print "", data.book_names[data.book_languages_index][i][1]
	
    exit()






if __name__ == '__main__':
    print "MiniBible " + str(data.version) + "." + str(data.build)
    
    app = QApplication(sys.argv)

    config = Configuration()
    config.Load()
    
    view = QDeclarativeView()
    view.setSource(QUrl.fromLocalFile('qml/main.qml'))
    root = view.rootObject()


    # instantiate the Python object
    qml_to_python = QML_to_Python_Interface()
    # expose the object to QML
    context = view.rootContext()
    context.setContextProperty("qml_to_python", qml_to_python)
    
    
    
    if(platform.machine().startswith('arm')): #we are on a N9/N950
	try:
	    os.makedirs(data.modules_path)
	    os.makedirs(data.modules_config_path)
	except:
	    pass
    
    
    data.modules_path = data.modules_path
    r = os.path.isdir(data.modules_path)    
    if(r==False): #module folder does not exist. are wo on a N9/N950?
      data.modules_path = data.modules_path_PC #try the PC version path
      data.modules_config_path = data.modules_config_path_PC #try the PC version path
      r = os.path.isdir(data.modules_path)    
      if(r==False):
	print "Neither the N9/N950 module path nor the PC module path seem to exist. I give up!"
      else: data.modulePathAvailable = True
    else: data.modulePathAvailable = True
    
    if(data.modulePathAvailable == True): print "Path to modules:", data.modules_path
    else: print "No module available"
    

    if(data.theme_inverted == True):
	root.themeInverted(True)
    
    
    translations = get_list_of_translations()
    for i in range(0, len(translations)):
      if(translations[i] == data.translation): #translation is available
	data.bibleSettingsValid = True
	break
	
	
    commandline()
	
	
    if(len(translations) > 0):
      root.loadListOfTranslationsFromPython(translations, data.translation, data.second_translation)    
      if(data.bibleSettingsValid == True):
	loadTranslation()
	if(data.book == ""): data.book = "Gen"
	if(data.chapter == ""): data.chapter = 1	
	#fillListOfBooks()
	
    if(data.bibleSettingsValid == True):
      print "Bible settings are valid, load text, settings"

      chapters = module.number_of_chapters_in_book(data.book)
      #verses=module.number_of_verses_in_chapter(data.book, data.chapter)      
      root.loadParametersfromPython(data.translation, data.second_translation, data.book, chapters, data.chapter, data.book_languages_index, data.font_sizes_index)
      fillBibleText()
      
    else: #load welcome screen
      print "Bible settings are NOT valid, load welcome screen"
      html = html_header()
      html = html + "<p class=title>MiniBible</p><p>This program is a bible viewer application</p>" + \
      "<p>There is not yet a bible translation installed. You will have to download at least one bible translation with the built in module manager (Menu > Settings > Add/Remove Translations). This is due copyright restrictions as well as to protect Christians in countries where they are persecuted because of their faith.</p>" + \
      "<p>For more information, please read the documentation (Menu > Help).</p>" + \
      "<p>Copyright 2011 by George Ruinelli</p>" + data.html_footer
      root.loadTextfromPython(data.html_header + html)
      root.loadParametersfromPython("", "", "", 0, 0, "", 1)
    
    
    
    if(platform.machine().startswith('arm')): view.showFullScreen()
    view.show()
    
    modulemanager = modulemanager.ModuleManager(root)

    
    app.exec_() #endless loop

    #app closing
    config.Write()
    